import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.text.ParseException;
import java.util.LinkedList;
import java.util.List;

import net.thehelper.blackhole.SC2LibOutput;
import net.thehelper.blackhole.voidparser.InStream;
import net.thehelper.blackhole.voidparser.VoidscriptParser;

public class Main {
	private static final String VERSION_NUM = "1.0.4 BETA";
	
	private static List<InStream> inStreamsToParse() {
		List<InStream> inStreams = new LinkedList<InStream>();

		File folder = new File("./void/");

		for (File file : folder.listFiles()) {
			try {
				if (file.getName().endsWith(".void")) {
					inStreams.add(new InStream(new BufferedReader(
							new FileReader(file))));
				}
			} catch (FileNotFoundException e) {
				System.out.println("Error opening file: " + file.getName());
			}
		}

		return inStreams;
	}

	public static void main(String[] args) {
		if (args.length>0 && args[0].contains("-version")){
			System.out.println("BlackHole written by Jesus4Lyf from thehelper.net.");
			System.out.println("Email: Jesus4Lyf@gmx.com");
			System.out.println("Version "+VERSION_NUM+".");
			return;
		}
		
		VoidscriptParser parser = new VoidscriptParser();

		for (InStream inStream : inStreamsToParse()) {

			try {
				parser.parseVoidCode(inStream);
			} catch (IOException e) {
				e.printStackTrace();
				return;
			} catch (ParseException e) {
				e.printStackTrace();
				return;
			}
		}
		
		parser.getWorkspace().compile();

		SC2LibOutput output;
		try {
			output = new SC2LibOutput();
		} catch (Exception e) {
			e.printStackTrace();
			return;
		}

		output.parseWorkspace(parser.getWorkspace());
		
		try {
			output.writeToFile(new File("Output.SC2Lib"));
		} catch (Exception e) {
			e.printStackTrace();
			return;
		}
	}
}
