package net.thehelper.blackhole;

import java.util.Stack;

import net.thehelper.galaxyutils.ParseUtils;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Text;

public class OutStream {
	private static final String NATIVE_ID_ACTION_CUSTOM_SCRIPT = "00000123";
	private static final String NATIVE_ID_EVENT_MAP_INIT = "00000120";
	private static final String GALAXY_NEW_LINE = "\n\t\t\t";
	private Document doc;
	private Element triggerData;
	private Element root;
	private StringBuilder initGalaxy = new StringBuilder("");
	private StringBuilder galaxy = new StringBuilder("");

	private Element standard;
	private Element name;
	private String libId;

	private Stack<Boolean> initMode = new Stack<Boolean>();

	public void addInitMode(boolean initMode) {
		this.initMode.push(initMode);
	}

	public void remoteInitMode() {
		this.initMode.pop();
	}

	private boolean isInitMode() {
		return initMode.peek();
	}

	public void appendGalaxyLine(String code) {
		if (isInitMode()) {
			initGalaxy.append(GALAXY_NEW_LINE);
			initGalaxy.append(code);
		} else {
			galaxy.append(GALAXY_NEW_LINE);
			galaxy.append(code);
		}
	}

	private String mapInitEventHex() {
		String hex = ParseUtils.generateRandomHex();
		Element mapInit = newElement("FunctionCall", hex);
		newElementPart(mapInit, "name");
		newTypeLibIdElement(mapInit, "function", "FunctionDef", "Ntve",
				NATIVE_ID_EVENT_MAP_INIT);
		return hex;
	}

	private String customScriptHex(String customScript) {
		String hex = ParseUtils.generateRandomHex();
		Element mapInit = newElement("FunctionCall", hex);
		newElementPart(mapInit, "name");
		newTypeLibIdElement(mapInit, "function", "FunctionDef", "Ntve",
				NATIVE_ID_ACTION_CUSTOM_SCRIPT);
		newElementPart(mapInit, "script").appendChild(
				newTextElement(customScript));
		return hex;
	}

	private void addInitFuncDeclaration() {
		String hex = ParseUtils.generateRandomHex();
		appendToRoot("Trigger", hex);
		Element initTrig = newElement("Trigger", hex);
		newElementPart(initTrig, "name").appendChild(newTextElement("Inj"));
		newElementPart(initTrig, "internal");
		newTypeLibIdElement(initTrig, "event", "FunctionCall", libId,
				mapInitEventHex());
		newTypeLibIdElement(initTrig, "action", "FunctionCall", libId,
				customScriptHex(hackCustomScript()));
	}

	public void seal() {
		addInitFuncDeclaration();
		addInitTrigger();
	}

	private void addInitTrigger() {
		String hex = ParseUtils.generateRandomHex();
		appendToRoot("Trigger", hex);
		Element initTrig = newElement("Trigger", hex);
		newElementPart(initTrig, "name").appendChild(newTextElement("Init"));
		newElementPart(initTrig, "internal");
		newTypeLibIdElement(initTrig, "event", "FunctionCall", libId,
				mapInitEventHex());
		newTypeLibIdElement(initTrig, "action", "FunctionCall", libId,
				customScriptHex(hackInit()));
	}

	private String hackCustomScript() {
		return GALAXY_NEW_LINE + "return true;}" + galaxy.toString()
				+ GALAXY_NEW_LINE + "bool BlackHoleInit(){" + GALAXY_NEW_LINE
				+ initGalaxy.toString() + GALAXY_NEW_LINE;
	}

	private String hackInit() {
		return GALAXY_NEW_LINE + "BlackHoleInit();" + GALAXY_NEW_LINE;
	}

	public void appendToRoot(String type, String id) {
		Element newElement = doc.createElement("item");
		newElement.setAttribute("type", type);
		newElement.setAttribute("library", libId);
		newElement.setAttribute("id", id);
		root.appendChild(newElement);
	}

	public Element newElement(String type, String id) {
		Element newElement = newElementPart(triggerData, "element");
		newElement.setAttribute("type", type);
		newElement.setAttribute("id", id);
		return newElement;
	}

	public Element newTypeLibIdElement(Element appendTo, String name,
			String type, String lib, String id) {
		Element newElement = newElementPart(appendTo, name);
		newElement.setAttribute("type", type);
		newElement.setAttribute("library", lib);
		newElement.setAttribute("id", id);
		return newElement;
	}

	public Element newTypeLibIdElement(Element appendTo, String name,
			String type, String id) {
		Element newElement = newElementPart(appendTo, name);
		newElement.setAttribute("type", type);
		newElement.setAttribute("library", libId);
		newElement.setAttribute("id", id);
		return newElement;
	}

	public Element newElementPart(Element appendTo, String name) {
		Element newElement = doc.createElement(name);
		appendTo.appendChild(newElement);
		return newElement;
	}

	public Text newTextElement(String text) {
		return doc.createTextNode(text);
	}

	public OutStream(Document doc) {
		this.initMode.push(false);
		this.doc = doc;
		triggerData = doc.createElement("TriggerData");
		standard = doc.createElement("standard");
		name = doc.createElement("name");
		root = doc.createElement("root");

		doc.appendChild(triggerData);
		triggerData.appendChild(standard);
		triggerData.appendChild(name);
		triggerData.appendChild(root);
	}

	public void setLibHeader(String libId, String libName) {
		this.libId = libId;
		standard.setAttribute("id", libId);
		name.appendChild(newTextElement(libName));
	}
}
