package net.thehelper.blackhole.model;

import java.text.ParseException;
import java.util.Collection;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import net.thehelper.blackhole.OutStream;
import net.thehelper.blackhole.model.modifier.Modifier;
import net.thehelper.galaxyutils.ParseUtils;

import org.w3c.dom.Element;

public class Function extends ScriptElement {
	private String name;
	private String finalIdentifier;
	private String returnType;
	private Collection<Variable> params;
	private Modifier mod;
	private List<ScriptElement> contents = new LinkedList<ScriptElement>();

	public List<ScriptElement> getContents() {
		return contents;
	}

	public String getName() {
		return name;
	}

	public Function(String returnType, String name,
			Collection<Variable> params, Modifier mod) throws ParseException {
		this.mod = mod;
		this.returnType = returnType;
		this.name = name;
		this.params = params;
	}

	@Override
	public void appendToXML(OutStream outStream) {
		String paramList = "";
		if (!params.isEmpty()) {
			for (Variable param : params) {
				paramList += "," + param.getType() + " " + param.getName();
			}
			paramList = paramList.substring(1);
		}

		outStream.appendGalaxyLine(returnType + " " + finalIdentifier + "("
				+ paramList + "){");
		for (ScriptElement statement : contents) {
			statement.appendToXML(outStream);
		}
		outStream.appendGalaxyLine("}");

		if (mod.isGui()) {
			// if (returnType == null) {
			appendActionDeclarationToXML(outStream);
			// } else {
			appendFunctionDeclarationToXML(outStream);
			// }
		}
	}

	private void appendFunctionDeclarationToXML(OutStream outStream) {
		String hex = ParseUtils.generateRandomHex();
		outStream.appendToRoot("FunctionDef", hex);
		Element func = outStream.newElement("FunctionDef", hex);
		outStream.newElementPart(func, "name").appendChild(
				outStream.newTextElement(finalIdentifier));
		outStream.newElementPart(func, "flagCall");
		outStream.newElementPart(func, "flagNative");
		if (returnType != null) {
			outStream.newElementPart(outStream.newElementPart(func, "return"),
					"type").setAttribute("val", returnType);
		}
		for (Variable param : params) {
			outStream.newTypeLibIdElement(func, "param", "ParamDef", param
					.appendAsParamToXML(outStream));
		}
	}

	private void appendActionDeclarationToXML(OutStream outStream) {
		String hex = ParseUtils.generateRandomHex();
		outStream.appendToRoot("FunctionDef", hex);
		Element func = outStream.newElement("FunctionDef", hex);
		outStream.newElementPart(func, "name").appendChild(
				outStream.newTextElement(finalIdentifier));
		outStream.newElementPart(func, "flagAction");
		outStream.newElementPart(func, "flagNative");
		if (returnType != null) {
			outStream.newElementPart(outStream.newElementPart(func, "return"),
					"type").setAttribute("val", returnType);
		}
		for (Variable param : params) {
			outStream.newTypeLibIdElement(func, "param", "ParamDef", param
					.appendAsParamToXML(outStream));
		}
	}

	@Override
	public void mapNames(Map<String, ScriptElement> nameMapping) {
		for (ScriptElement element : contents) {
			element.mapNames(nameMapping);
		}
	}

	@Override
	public String performMapping(String input) {
		return input.replaceAll(matchRegex(), finalIdentifier + "(");
	}

	private String matchRegex() {
		return name + "[ 	]*\\(";
	}

	@Override
	public void constructNameMapping(Map<String, ScriptElement> nameMapping) {
		nameMapping.put(matchRegex(), this);
	}

	@Override
	public void setParent(ScriptElement parent) {
		super.setParent(parent);
		if (mod.isPublic()) {
			this.finalIdentifier = ParseUtils.makeIdentifierPublic(getParent()
					.getName(), name);
		} else {
			this.finalIdentifier = ParseUtils.makeIdentifierPrivate(getParent()
					.getName(), name);
		}
	}

	@Override
	public void resolveParents() {
		for (ScriptElement element : contents) {
			element.setParent(this);
			element.resolveParents();
		}
	}
}
