package net.thehelper.blackhole.model;

import java.text.ParseException;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import net.thehelper.blackhole.OutStream;

public class Library extends ScriptElement {
	private String name;

	private Map<String, ScriptElement> internalIdentifiers = new HashMap<String, ScriptElement>();
	private List<ScriptElement> unnamedParts = new LinkedList<ScriptElement>();

	public List<ScriptElement> getUnnamedParts() {
		return unnamedParts;
	}

	public Map<String, ScriptElement> getInternalIdentifiers() {
		return internalIdentifiers;
	}

	public Library(String name) throws ParseException {
		this.name = name;
	}

	public void appendToXML(OutStream outStream) {
		for (ScriptElement element : internalIdentifiers.values()) {
			element.appendToXML(outStream);
		}
		for (ScriptElement element : unnamedParts) {
			element.appendToXML(outStream);
		}
	}

	public String getName() {
		return name;
	}

	@Override
	public void constructNameMapping(Map<String, ScriptElement> nameMapping) {
		for (ScriptElement element : internalIdentifiers.values()) {
			element.constructNameMapping(nameMapping);
		}
		for (ScriptElement element : unnamedParts) {
			element.constructNameMapping(nameMapping);
		}
	}

	@Override
	public void mapNames(Map<String, ScriptElement> nameMapping) {
		for (ScriptElement element : internalIdentifiers.values()) {
			element.mapNames(nameMapping);
		}
		for (ScriptElement element : unnamedParts) {
			element.mapNames(nameMapping);
		}
	}
	
	public void resolveIdentifiers(){
		Map<String, ScriptElement> nameMapping = new HashMap<String, ScriptElement>();
		constructNameMapping(nameMapping);
		mapNames(nameMapping);
	}

	@Override
	public String performMapping(String input) {
		return input;
	}

	@Override
	public void resolveParents() {
		for (ScriptElement element : internalIdentifiers.values()) {
			element.setParent(this);
			element.resolveParents();
		}
	}
}
