package net.thehelper.blackhole.model;

import java.text.ParseException;
import java.util.LinkedHashMap;
import java.util.Map;

import net.thehelper.blackhole.OutStream;
import net.thehelper.blackhole.model.modifier.Modifier;
import net.thehelper.galaxyutils.ParseUtils;

public class StructType extends ScriptElement {
	private String name;
	private Map<String, Variable> members = new LinkedHashMap<String, Variable>();
	private Modifier mod;
	private String finalIdentifier;

	public String getName() {
		return name;
	}

	public Map<String, Variable> getMembers() {
		return members;
	}

	public StructType(String name, Modifier mod) throws ParseException {
		this.name = name;
		this.mod = mod;
	}

	@Override
	public void appendToXML(OutStream outStream) {
		outStream.appendGalaxyLine("struct " + finalIdentifier + "{");
		for (Variable member : members.values()) {
			member.appendToXML(outStream);
		}
		outStream.appendGalaxyLine("};"); // <-- stupid ';'.
	}

	@Override
	public void constructNameMapping(Map<String, ScriptElement> nameMapping) {
		nameMapping.put(matchRegex(), this);
	}

	@Override
	public void mapNames(Map<String, ScriptElement> nameMapping) {
		// TODO Auto-generated method stub

	}

	@Override
	public void setParent(ScriptElement parent) {
		super.setParent(parent);
		if (mod.isPublic()) {
			this.finalIdentifier = ParseUtils.makeIdentifierPublic(getParent()
					.getName(), name);
		} else {
			this.finalIdentifier = ParseUtils.makeIdentifierPrivate(getParent()
					.getName(), name);
		}
	}

	@Override
	public String performMapping(String input) {
		return input.replaceAll(matchRegex(), finalIdentifier + " ");
	}

	private String matchRegex() {
		return name + "[ 	]+";
	}

	@Override
	public void resolveParents() {
		for (ScriptElement element : members.values()) {
			element.setParent(this);
			element.resolveParents();
		}
	}
}
