package net.thehelper.blackhole.model;

import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import net.thehelper.blackhole.OutStream;
import net.thehelper.blackhole.model.modifier.Modifier;
import net.thehelper.galaxyutils.ParseUtils;

import org.w3c.dom.Element;

public class Variable extends ScriptElement {
	private String name;
	private String type;
	private String initialValue;
	private List<Integer> arrayDimensions;
	private Modifier mod;
	private String finalIdentifier;

	public String getName() {
		return name;
	}

	public String getType() {
		return type;
	}

	public String initialValue() {
		return initialValue;
	}

	public Variable(String name, String type, List<Integer> arrayDimensions,
			String initialValue, Modifier mod) {
		this.name = name;
		this.type = type;
		this.arrayDimensions = arrayDimensions;
		this.initialValue = initialValue;
		this.mod = mod;
	}

	@Override
	public void appendToXML(OutStream outStream) {
		StringBuilder result = new StringBuilder(getType());
		if (arrayDimensions != null) {
			for (Integer dimension : arrayDimensions) {
				result.append("[" + dimension.toString() + "]");
			}
		}
		result.append(" " + finalIdentifier);
		if (initialValue != null) {
			result.append("=" + initialValue);
		}
		result.append(";");
		outStream.appendGalaxyLine(result.toString());
	}

	public String appendAsParamToXML(OutStream outStream) {
		String hex = ParseUtils.generateRandomHex();
		Element func = outStream.newElement("ParamDef", hex);
		outStream.newElementPart(func, "name").appendChild(
				outStream.newTextElement(finalIdentifier));
		outStream.newElementPart(outStream.newElementPart(func, "paramType"),
				"type").setAttribute("val", type);
		// TODO: defaults.. <default type="Param" library="MLIB" id="336CA788"/>
		return hex;
	}

	@Override
	public void setParent(ScriptElement parent) {
		super.setParent(parent);
		if (parent instanceof Library) {
			if (mod.isPublic()) {
				this.finalIdentifier = ParseUtils.makeIdentifierPublic(
						getParent().getName(), name);
			} else {
				this.finalIdentifier = ParseUtils.makeIdentifierPrivate(
						getParent().getName(), name);
			}
		} else {
			finalIdentifier = name;
		}
	}

	@Override
	public void constructNameMapping(Map<String, ScriptElement> nameMapping) {
		nameMapping.put(matchRegex(), this);
	}

	private String matchRegex() {
		return name;
	}

	@Override
	public void mapNames(Map<String, ScriptElement> nameMapping) {
		for (Entry<String, ScriptElement> mapping : nameMapping.entrySet()) {
			if (ParseUtils.containsRegex(initialValue, mapping.getKey())) {
				initialValue = mapping.getValue().performMapping(initialValue);
			}
		}
	}

	@Override
	public String performMapping(String input) {
		return input.replaceAll(matchRegex(), finalIdentifier);
	}

	@Override
	public void resolveParents() {
	}
}
