package net.thehelper.blackhole.model;

import java.util.HashMap;
import java.util.Map;

import net.thehelper.blackhole.OutStream;

public class Workspace extends ScriptElement {
	private Map<String, Library> libraries = new HashMap<String, Library>();

	public Map<String, Library> getLibraries() {
		return libraries;
	}

	public void appendToXML(OutStream outStream) {
		String libId = "J4BH";
		String libName = "BlackHole Compilation";
		if (libraries.size() == 1) {
			libId = libraries.values().iterator().next().getName();
			libName = libId;
		}
		outStream.setLibHeader(libId, libName);

		for (Library lib : libraries.values()) {
			lib.appendToXML(outStream);
		}
	}

	public void resolveParents() {
		for (Library lib : libraries.values()) {
			lib.setParent(this);
			lib.resolveParents();
		}
	}

	public void resolveIdentifiers() {
		for (Library lib : libraries.values()) {
			lib.resolveIdentifiers();
		}
	}

	public void compile() {
		resolveParents();
		resolveIdentifiers();
	}

	@Override
	public void constructNameMapping(Map<String, ScriptElement> nameMapping) {
		throw new UnsupportedOperationException(
				"Workspaces cannot perform mappings.");
	}

	@Override
	public String getName() {
		throw new UnsupportedOperationException(
				"Cannot get the name of a Workspace.");
	}

	@Override
	public void mapNames(Map<String, ScriptElement> nameMapping) {
		throw new UnsupportedOperationException("Workspaces cannot map names.");
	}

	@Override
	public String performMapping(String input) {
		throw new UnsupportedOperationException(
				"Workspaces cannot perform mappings.");
	}
}
