package net.thehelper.blackhole.voidparser;

import java.io.BufferedReader;
import java.io.EOFException;
import java.io.IOException;

import net.thehelper.galaxyutils.GalaxyUtils;
import static net.thehelper.galaxyutils.GalaxyUtils.STATEMENT_ENDERS;

public class InStream {
	private BufferedReader reader;

	public InStream(BufferedReader reader) {
		this.reader = reader;
	}

	private int readCount = 0;

	private String readFileLine() throws IOException {
		readCount++;
		String line = reader.readLine();
		if (line == null) {
			reader.close();
		}
		return line;
	}

	private String readLine() throws IOException {
		String line = readFileLine();
		if (line != null) {
			boolean cont = true;
			while (cont) {
				cont = false;
				line = GalaxyUtils.stripLineComments(line);
				line = GalaxyUtils.stripBlockComments(line);
				while (line.contains("/*")) {
					try {
						line = GalaxyUtils.stripBlockComments(line
								+ readFileLine());
					} catch (NullPointerException e) {
						throw new EOFException("Block comment was not closed.");
					}
					cont = true;
				}
			}
		}
		return line;
	}

	public int getReadLineCount() {
		return readCount;
	}

	private String carryOver = "";

	private void appendToCarryOver(String append) {
		carryOver = (carryOver += " " + append).trim();
	}

	private int nextStatementEnd() {
		int first = carryOver.length() + 1;
		int index;

		for (char statementEnder : STATEMENT_ENDERS) {
			index = carryOver.indexOf(statementEnder);
			if (index >= 0 && index < first) {
				first = index;
			}
		}

		return first;
	}

	private int startingLine = 0;

	public int getLastStatementStartLine() {
		return startingLine;
	}

	private static final String MUST_READ_STATEMENT = "";

	private String nextStatement = MUST_READ_STATEMENT;

	public String popStatement() throws IOException {
		if (nextStatement == MUST_READ_STATEMENT) {
			return readStatement();
		}
		String poppedStatement = nextStatement;
		nextStatement = MUST_READ_STATEMENT;
		return poppedStatement;
	}

	public String peekStatement() throws IOException {
		if (nextStatement == MUST_READ_STATEMENT) {
			nextStatement = readStatement();
			return nextStatement;
		}
		return nextStatement;
	}

	private String readStatement() throws IOException, EOFException {
		startingLine = readCount + 1;
		int endOfStatement = nextStatementEnd();
		while (endOfStatement == carryOver.length() + 1) {
			String line = readLine();
			if (line == null) {
				if (carryOver.isEmpty()) {
					return null;
				} else {
					throw new EOFException(
							"File ended with unclosed statement: "
									+ carryOver.trim());
				}
			}
			appendToCarryOver(line);
			endOfStatement = nextStatementEnd();
		}
		String result = carryOver.substring(0, endOfStatement + 1);
		carryOver = carryOver.substring(endOfStatement + 1).trim();
		return result;
	}
}
