package net.thehelper.blackhole.voidparser;

import java.io.IOException;
import java.text.ParseException;
import java.util.Collection;

import net.thehelper.blackhole.model.Function;
import net.thehelper.blackhole.model.ScriptElement;
import net.thehelper.blackhole.model.Variable;
import net.thehelper.blackhole.model.modifier.GuiModifier;
import net.thehelper.blackhole.model.modifier.Modifier;
import net.thehelper.galaxyutils.GalaxyUtils;
import net.thehelper.galaxyutils.ParseUtils;

public class VoidscriptFunctionParser extends VoidscriptElementParser {

	public VoidscriptFunctionParser() {
		addSubParser(new VoidscriptBasicStatementParser());
		addSubParser(new VoidscriptComplexStatementParser());
		addModifierParser(new GuiModifier());
		addModifierParser(new PublicModifier());
	}

	@Override
	public Function parse(InStream inStream) throws ParseException, IOException {
		if (inStream.peekStatement().matches(".*\\(.*\\).*\\{")) {
			String statement = inStream.popStatement();
			int lineNum = inStream.getLastStatementStartLine();

			Modifier mod = new Modifier();
			statement = parseModifier(mod, statement);

			String[] whitespaceSplit = ParseUtils.splitWhitespace(statement
					.replace("(", " ("));
			String name = whitespaceSplit[1];
			GalaxyUtils.verifyValidIdentifier(name, statement, lineNum);
			String returnType = whitespaceSplit[0];
			Collection<Variable> params = ParseUtils.getFunctionParams(
					statement, lineNum);
			Function newFunc = new Function(returnType, name, params, mod);

			while (true) {
				ScriptElement newFunctionPart = subParse(inStream);
				if (newFunctionPart == null) {
					statement = inStream.popStatement();
					lineNum = inStream.getLastStatementStartLine();
					if (statement == null) {
						throw new ParseException("Function "
								+ newFunc.getName() + " was not closed!",
								lineNum);
					}
					if (statement.equals("}")) {
						return newFunc;
					}
					throw new ParseException("Unexpected statement: "
							+ statement, lineNum);
				}
				newFunc.getContents().add(newFunctionPart);
			}
		}
		return null;
	}

}
