package net.thehelper.blackhole.voidparser;

import java.io.IOException;
import java.text.ParseException;

import net.thehelper.blackhole.model.Library;
import net.thehelper.blackhole.model.ScriptElement;
import net.thehelper.galaxyutils.ParseUtils;

public class VoidscriptLibraryParser extends VoidscriptElementParser {

	public VoidscriptLibraryParser() {
		addSubParser(new VoidscriptInitParser());
		addSubParser(new VoidscriptFunctionParser());
		addSubParser(new VoidscriptStructTypeParser());
		addSubParser(new VoidscriptVariableParser());
	}

	private static void assertValidLibraryDeclaration(String statement,
			String[] decSplit, int lineNum) throws ParseException {
		if (!statement.endsWith("{")) {
			throw new ParseException("Expected { after library on line: "
					+ statement, lineNum);
		}

		if (!decSplit[0].equals("library")) {
			throw new ParseException("Expected library on line: " + statement,
					lineNum);
		}

		if (decSplit.length > 2) {
			throw new ParseException("Unexpected " + decSplit[2] + " on line: "
					+ statement, lineNum);
		}

		if (decSplit[1].length() != 4) {
			throw new ParseException(
					"Library names must be 4 characters in length", lineNum);
		}
	}

	@Override
	public ScriptElement parse(InStream inStream) throws ParseException,
			IOException {
		if (inStream.peekStatement().startsWith("library")) {
			String statement = inStream.popStatement();
			int lineNum = inStream.getLastStatementStartLine();

			String[] split = ParseUtils.splitWhitespace(ParseUtils
					.stripStatementEnder(statement));

			assertValidLibraryDeclaration(statement, split, lineNum);
			Library newLib = new Library(split[1]);

			while (true) {
				statement = inStream.peekStatement();
				lineNum = inStream.getLastStatementStartLine();
				ScriptElement newLibraryPart = subParse(inStream);
				if (newLibraryPart == null) {
					statement = inStream.popStatement();
					lineNum = inStream.getLastStatementStartLine();
					if (statement == null) {
						throw new ParseException("Library " + newLib.getName()
								+ " was not closed!", lineNum);
					}
					if (statement.endsWith("}")) {
						return newLib;
					}
					throw new ParseException("Unexpected statement: "
							+ statement, lineNum);
				}
				try {
					if (newLib.getInternalIdentifiers().put(
							newLibraryPart.getName(), newLibraryPart) != null) {
						throw new ParseException("Identifier "
								+ newLibraryPart.getName()
								+ " redeclared in line: " + statement, lineNum);
					}
				} catch (UnsupportedOperationException e) {
					newLib.getUnnamedParts().add(newLibraryPart);
				}
			}
		}
		return null;
	}
}
