package net.thehelper.blackhole.voidparser;

import java.io.IOException;
import java.text.ParseException;

import net.thehelper.blackhole.model.StructType;
import net.thehelper.blackhole.model.Variable;
import net.thehelper.blackhole.model.modifier.Modifier;
import net.thehelper.galaxyutils.GalaxyUtils;
import net.thehelper.galaxyutils.ParseUtils;

public class VoidscriptStructTypeParser extends VoidscriptElementParser {

	public VoidscriptStructTypeParser() {
		addSubParser(new VoidscriptVariableParser());
		addModifierParser(new PublicModifier());
	}

	@Override
	public StructType parse(InStream inStream) throws ParseException,
			IOException {
		if (inStream.peekStatement().matches(".*struct .+\\{")) {
			String statement = inStream.popStatement();
			int lineNum = inStream.getLastStatementStartLine();

			Modifier mod = new Modifier();
			statement = parseModifier(mod, statement);

			String[] whitespaceSplit = ParseUtils.splitWhitespace(ParseUtils
					.stripStatementEnder(statement));
			String name = whitespaceSplit[1];
			GalaxyUtils.verifyValidIdentifier(name, statement, lineNum);
			if (!whitespaceSplit[0].equals("struct")) {
				throw new ParseException("Unexpected " + whitespaceSplit[0]
						+ " in line " + statement + ".", lineNum);
			}
			StructType newStructType = new StructType(name, mod);

			while (true) {
				Variable newMember = (Variable) subParse(inStream);
				if (newMember == null) {
					statement = inStream.popStatement();
					lineNum = inStream.getLastStatementStartLine();
					if (statement == null) {
						throw new ParseException("Struct "
								+ newStructType.getName() + " was not closed!",
								lineNum);
					}
					if (statement.equals("}")) {
						return newStructType;
					}
					throw new ParseException("Unexpected statement: "
							+ statement, lineNum);
				}
				if (newStructType.getMembers().put(newMember.getName(),
						newMember) != null) {
					throw new ParseException("Struct member "
							+ newMember.getName() + " redeclared in line: "
							+ statement, lineNum);
				}
			}
		}
		return null;
	}

}
