package net.thehelper.blackhole.voidparser;

import java.io.IOException;
import java.text.ParseException;
import java.util.List;

import net.thehelper.blackhole.model.ScriptElement;
import net.thehelper.blackhole.model.Variable;
import net.thehelper.blackhole.model.modifier.Modifier;
import net.thehelper.galaxyutils.GalaxyUtils;
import net.thehelper.galaxyutils.ParseUtils;

public class VoidscriptVariableParser extends VoidscriptElementParser {
	public VoidscriptVariableParser() {
		addModifierParser(new PublicModifier());
	}

	public Variable parseDeclaration(String dec, String statement, int lineNum)
			throws ParseException {
		dec = ParseUtils.stripStatementEnder(dec).replace("=[ 	]*",
				" =");

		Modifier mod = new Modifier();
		dec = parseModifier(mod, dec);

		String[] whitespaceSplit = ParseUtils.splitWhitespace(dec);
		String name = whitespaceSplit[1];
		String type = whitespaceSplit[0];
		GalaxyUtils.verifyValidIdentifier(name, statement, lineNum);
		GalaxyUtils.verifyValidIdentifier(type, statement, lineNum);

		int index = dec.indexOf("=");
		String initialValue = null;
		List<Integer> arrayDimensions = null;
		if (index == -1) {
			arrayDimensions = ParseUtils.getArrayDimensions(dec, statement,
					lineNum);
		} else {
			arrayDimensions = ParseUtils.getArrayDimensions(dec.substring(0,
					index), statement, lineNum);
			initialValue = dec.substring(index);
			if (initialValue.equals("=")) {
				throw new ParseException("Variable " + name
						+ " contained '=' followed by nothing.", lineNum);
			}
			initialValue = initialValue.substring(1);
		}
		Variable newVar = new Variable(name, type, arrayDimensions,
				initialValue, mod);
		return newVar;
	}

	@Override
	public ScriptElement parse(InStream inStream) throws ParseException,
			IOException {
		if (inStream.peekStatement().matches(".+ .+;")
				&& !inStream.peekStatement().split("=")[0].contains("(")) {
			String statement = inStream.popStatement();
			int lineNum = inStream.getLastStatementStartLine();
			Variable newVar = parseDeclaration(statement, statement, lineNum);
			return newVar;
		}
		return null;
	}

}
