package net.thehelper.galaxyutils;

import java.text.ParseException;
import java.util.Arrays;
import java.util.List;

public class GalaxyUtils {
	public static final List<String> NATIVE_TYPE_LIST = Arrays
			.asList(new String[] { "wavetarget", "waveinfo", "wave", "unitref",
					"unitgroup", "unitfilter", "unit", "trigger",
					"transmissionsource", "timer", "text", "string",
					"soundlink", "revealer", "region", "point", "playergroup",
					"order", "marker", "int", "handle", "fixed", "doodad",
					"color", "char", "camerainfo", "byte", "bool", "bank",
					"animtarget", "aifilter", "actorscope", "actor", "abilcmd",
					"void" });

	public static void verifyValidIdentifier(String name, String line, int lineNum)
			throws ParseException {
		if (name.length() == 0) {
			throw new ParseException("Empty identifier specified on line: "+line, lineNum);
		}

		if (!Character.isLetter(name.charAt(0))) {
			throw new ParseException("Identifier " + name
					+ " starts with a number on line: "+line, lineNum);
		}

		for (int i = 1; i < name.length(); i++) {
			if (!Character.isLetterOrDigit(name.charAt(i))
					&& !name.substring(i, i + 1).equals("_")) {
				throw new ParseException("Identifier " + name
						+ " contains invalid character: " + name.charAt(i)
						+ " at line: "+line, lineNum);
			}
		}
	}

	private static final char DOUBLE_QUOTE = "\"".charAt(0);
	private static final char SINGLE_QUOTE = "'".charAt(0);

	public static boolean isStringIndexInQuotes(String source, int index) {
		char status = 0;

		char[] chars = source.toCharArray();
		for (int i = 0; i < chars.length; i++) {
			if (i == index) {
				return status != 0;
			}

			if (status != 0) {
				if (chars[i] == status) {
					status = 0;
				}
			} else {
				if (chars[i] == DOUBLE_QUOTE) {
					status = DOUBLE_QUOTE;
				} else if (chars[i] == SINGLE_QUOTE) {
					status = SINGLE_QUOTE;
				}
			}
		}
		return status != 0;
	}

	public static String stripLineComments(String line) {
		int index = line.indexOf("//");
		while (index != -1) {
			if (!isStringIndexInQuotes(line, index)) {
				return line.substring(0, index).trim();
			}
			index = line.indexOf("//", index + 1);
		}
		return line;
	}

	public static String stripBlockComments(String str) {
		return str.replaceAll("/\\*(?:.|[\\n\\r])*?\\*/", "").trim();
	}

	public static final char[] STATEMENT_ENDERS = new char[] { ";".charAt(0),
			"{".charAt(0), "}".charAt(0) };
}
