package net.thehelper.galaxyutils;

import static net.thehelper.galaxyutils.GalaxyUtils.STATEMENT_ENDERS;

import java.text.ParseException;
import java.util.Collection;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Random;

import net.thehelper.blackhole.model.Variable;
import net.thehelper.blackhole.voidparser.VoidscriptVariableParser;

public class ParseUtils {
	private static final VoidscriptVariableParser VARIABLE_PARSER = new VoidscriptVariableParser();
	public static final String GUI_MODIFIER = "gui";
	public static final String PUBLIC_MODIFIER = "public";

	public static String[] splitWhitespace(String source) {
		return source.split("[	 ]+");
	}

	public static String stripStatementEnder(String statement) {
		int index = -1;
		for (char ender : STATEMENT_ENDERS) {
			int thisIndex = statement.lastIndexOf(ender);
			if (thisIndex > index) {
				index = thisIndex;
			}
		}
		if (index == -1) {
			return statement;
		}
		return statement.substring(0, index).trim();
	}

	public static String truncFirstWord(String str) {
		int index = splitWhitespace(str)[0].length();
		return str.substring(index, str.length()).trim();
	}

	public static String makeIdentifierPrivate(String libName, String identifier) {
		return "BH__" + libName + "__" + identifier;
	}

	public static String makeIdentifierPublic(String libName, String identifier) {
		return libName + "_" + identifier;
	}

	public static Collection<Variable> getFunctionParams(String statement,
			int lineNum) throws ParseException {
		Map<String, Variable> params = new LinkedHashMap<String, Variable>();
		if (!statement.matches(".*\\(.*\\).*")) {
			throw new ParseException("Could not find parameters for line: "
					+ statement, lineNum);
		}
		String[] split = statement.substring(statement.indexOf("(") + 1,
				statement.indexOf(")")).split(",");
		if (split.length == 1 && split[0].trim().isEmpty()) {
			return params.values();
		}
		for (String paramDec : split) {
			Variable paramVar = VARIABLE_PARSER.parseDeclaration(paramDec
					.trim(), statement, lineNum);
			if (params.put(paramVar.getName(), paramVar) != null) {
				throw new ParseException("Parameter identifier "
						+ paramVar.getName() + " reused in line: " + statement,
						lineNum);
			}
		}
		return params.values();
	}

	public static List<Integer> getArrayDimensions(String dec,
			String statement, int lineNum) throws ParseException {
		if (!dec.matches(".*\\[.*\\].*")) {
			return null;
		}
		String[] split = dec.split("\\][ 	]*\\[");
		List<Integer> dimensions = new LinkedList<Integer>();
		for (String dimension : split) {
			dimension = dimension.replaceAll(".*\\[", "").replaceAll("\\].*",
					"").trim();
			try {
				dimensions.add(new Integer(dimension));
			} catch (Exception e) {
				throw new ParseException(
						"Invalid array dimension in statement: " + statement,
						lineNum);
			}
		}
		return dimensions;
	}

	public static String generateRandomHex() {
		Random random = new Random();
		return Integer.toHexString(random.nextInt()).toUpperCase();
	}

	public static boolean containsRegex(String statement, String regex) {
		return statement.matches(".*" + regex + ".*");
	}
}
