package net.thehelper.galaxyutils;

import static org.junit.Assert.assertArrayEquals;

import java.text.ParseException;
import java.util.Collection;
import java.util.Iterator;

import junit.framework.TestCase;
import net.thehelper.blackhole.model.Variable;

public class ParseUtilsTest extends TestCase {
	public void testSplitWhitespace() {
		assertArrayEquals(new String[] { "monkeydog" }, ParseUtils
				.splitWhitespace("monkeydog"));
		assertArrayEquals(new String[] { "dog", "cat", "monkey" }, ParseUtils
				.splitWhitespace("dog cat	monkey"));
		assertArrayEquals(new String[] { "dog", "cat", "monkey" }, ParseUtils
				.splitWhitespace("dog  cat		monkey"));
		assertArrayEquals(new String[] { "dog", "cat", "monkey" }, ParseUtils
				.splitWhitespace("dog	 cat 	monkey"));
	}

	public void testStripStatementEnder() {
		assertEquals("dog", ParseUtils.stripStatementEnder("dog"));
		assertEquals("dog", ParseUtils.stripStatementEnder("dog;"));
		assertEquals("dog", ParseUtils.stripStatementEnder("dog{"));
		assertEquals("dog", ParseUtils.stripStatementEnder("dog}"));
		assertEquals("dog", ParseUtils.stripStatementEnder("dog; "));
		assertEquals("dog", ParseUtils.stripStatementEnder("dog ; "));
		assertEquals("dog", ParseUtils.stripStatementEnder("dog ;"));
	}

	public void testGetFunctionParams() throws ParseException {
		Collection<Variable> current;
		Iterator<Variable> iter;

		current = ParseUtils.getFunctionParams(
				"myFunc(dog woofles, cat mittens){", 0);
		iter = current.iterator();

		assertEquals(2, current.size());
		assertEquals("dog", iter.next().getType());
		assertEquals("mittens", iter.next().getName());
		
		current = ParseUtils.getFunctionParams(
				"(dog woofles,cat mittens)", 0);
		iter = current.iterator();

		assertEquals(2, current.size());
		assertEquals("woofles", iter.next().getName());
		assertEquals("cat", iter.next().getType());
	}
	
	public void testGenerateRandomHex() {
		assertEquals(8, ParseUtils.generateRandomHex().length());
		assertEquals(8, ParseUtils.generateRandomHex().length());
		assertEquals(8, ParseUtils.generateRandomHex().length());
	}
}
